import { __awaiter } from 'tslib';
import { Subject } from 'rxjs';
import { DragController, DropController, LocalStorage, SessionStorage } from '@anjuna/core';
import { InjectionToken, Inject, Optional, Directive, ElementRef, Input, NgZone, TemplateRef, ViewContainerRef, ContentChildren, Attribute, EventEmitter, HostListener, Output, Renderer2, ChangeDetectorRef, HostBinding, Injector, Self, LOCALE_ID, Pipe, Injectable, NgModule, defineInjectable } from '@angular/core';
import { FormGroupDirective, NgControl, NgForm, NG_VALUE_ACCESSOR, NG_VALIDATORS, FormControl } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const LANGUAGE_CODE = new InjectionToken('LanguageCode');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 */
class BaseTranslationPipe {
    /**
     * @param {?} language
     */
    constructor(language) {
        this.language = language;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        if (!this.translations[this.language]) {
            console.warn(`Translation file for '${this.language}' does not exist.`);
            return value;
        }
        return this.translations[this.language][value] || value;
    }
}
/** @nocollapse */
BaseTranslationPipe.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [LANGUAGE_CODE,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@description A wrapper around the HTML 5 Drag API.
 */
class DraggableDirective {
    /**
     * @param {?} element
     */
    constructor(element) {
        this.element = element;
        this.dataFn = () => this.getData();
        this.controller = new DragController(this.element.nativeElement, this.dataFn).connect();
    }
    /**
     * @private
     * @return {?}
     */
    getData() {
        return this.anjDraggable;
    }
}
DraggableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjDraggable]'
            },] }
];
/** @nocollapse */
DraggableDirective.ctorParameters = () => [
    { type: ElementRef }
];
DraggableDirective.propDecorators = {
    anjDraggable: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@description A wrapper around the HTML 5 Drop API.
 */
class DroppableDirective {
    /**
     * @param {?} element
     */
    constructor(element) {
        this.element = element;
        this.controller = new DropController(this.element.nativeElement).connect();
    }
}
DroppableDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjDroppable]'
            },] }
];
/** @nocollapse */
DroppableDirective.ctorParameters = () => [
    { type: ElementRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DomOutlet {
    /**
     * @param {?} targetElement
     */
    constructor(targetElement) {
        this.targetElement = targetElement;
    }
    /**
     * @template C
     * @param {?} template
     * @return {?}
     */
    attachTemplate(template) {
        if (this.template.zone.isStable) {
            this._viewRef = this.template.zone.run(() => {
                return template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
            });
        }
        else {
            this._viewRef = template.viewContainerRef.createEmbeddedView(template.templateRef, template.context);
        }
        this._viewRef.rootNodes.forEach(rootNode => this.targetElement.appendChild(rootNode));
        this._viewRef.detectChanges();
        this.setDisposeFn((() => {
            /** @type {?} */
            const index = template.viewContainerRef.indexOf(this._viewRef);
            if (index !== -1) {
                template.viewContainerRef.remove(index);
            }
        }));
        return this._viewRef;
    }
    /**
     * @param {?} context
     * @return {?}
     */
    updateContext(context) {
        for (const prop in context) {
            if (context.hasOwnProperty(prop)) {
                this._viewRef.context[prop] = context[prop];
            }
        }
        this._viewRef.detectChanges();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    setDisposeFn(fn) {
        this.disposeFn = fn;
    }
    /**
     * @param {?} template
     * @return {?}
     */
    attach(template) {
        this.template = template;
        return this.attachTemplate(template);
    }
    /**
     * @return {?}
     */
    detach() {
        if (this.template) {
            this.template.setAttachedHost(null);
            this.template = null;
        }
        this.invokeDisposeFn();
    }
    /**
     * @param {?=} removeParent
     * @return {?}
     */
    dispose(removeParent = true) {
        this.disposeFn();
        if (removeParent && this.targetElement.parentNode != null) {
            this.targetElement.parentNode.removeChild(this.targetElement);
        }
        return undefined;
    }
    /**
     * @private
     * @return {?}
     */
    invokeDisposeFn() {
        if (this.disposeFn) {
            this.disposeFn();
            this.disposeFn = null;
        }
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Template {
    /**
     * @param {?} templateDirective
     * @param {?=} context
     */
    constructor(templateDirective, context) {
        this.templateDirective = templateDirective;
        this.context = context;
    }
    /**
     * @return {?}
     */
    get origin() {
        return this.templateDirective.template.elementRef;
    }
    /**
     * @return {?}
     */
    get templateRef() {
        return this.templateDirective.template;
    }
    /**
     * @return {?}
     */
    get viewContainerRef() {
        return this.templateDirective.viewContainerRef;
    }
    /**
     * @return {?}
     */
    get zone() {
        return this.templateDirective.zone;
    }
    /**
     * @param {?} host
     * @param {?=} context
     * @return {?}
     */
    attach(host, context = this.context) {
        this.context = context;
        this.host = host;
        return host.attach(this);
    }
    /**
     * @return {?}
     */
    detach() {
        this.context = undefined;
        if (this.host) {
            /** @type {?} */
            const host = this.host;
            this.host = null;
            host.detach();
        }
    }
    /**
     * @param {?} host
     * @return {?}
     */
    setAttachedHost(host) {
        this.host = host;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TemplateDirective {
    /**
     * @param {?} _template
     * @param {?} _viewRef
     * @param {?} _zone
     */
    constructor(_template, _viewRef, _zone) {
        this._template = _template;
        this._viewRef = _viewRef;
        this._zone = _zone;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    set anjTemplateOf(name) {
        this._name = name;
    }
    /**
     * @return {?}
     */
    get name() {
        return this._name;
    }
    /**
     * @return {?}
     */
    get template() {
        return this._template;
    }
    /**
     * @return {?}
     */
    get viewContainerRef() {
        return this._viewRef;
    }
    /**
     * @return {?}
     */
    get zone() {
        return this._zone;
    }
}
TemplateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjTemplate]'
            },] }
];
/** @nocollapse */
TemplateDirective.ctorParameters = () => [
    { type: TemplateRef },
    { type: ViewContainerRef },
    { type: NgZone }
];
TemplateDirective.propDecorators = {
    anjTemplateOf: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class AnjTable {
    /**
     * @param {?} elRef
     */
    constructor(elRef) {
        this.outletRows = [];
        this.element = elRef.nativeElement;
        this.element.renderer = this;
    }
    /**
     * @private
     * @param {?} row
     * @param {?} index
     * @param {?=} column
     * @return {?}
     */
    static rowContext(row, index, column) {
        return {
            $implicit: row,
            column,
            index
        };
    }
    /**
     * @private
     * @param {?} outlet
     * @param {?} context
     * @return {?}
     */
    static updateContext(outlet, context) {
        outlet.updateContext(context);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.outletRows.map((rowMap) => {
            for (const templateName in rowMap) {
                if (rowMap[templateName]) {
                    rowMap[templateName] = rowMap[templateName].dispose();
                }
            }
            return rowMap;
        });
    }
    /* These next three methods are a mess of duplicated logic - they will probably stay that way until feature complete, then I'll clean em up */
    /**
     * @param {?} cell
     * @param {?} offset
     * @param {?} col
     * @param {?} row
     * @param {?} rowIndex
     * @param {?} template
     * @return {?}
     */
    cellTemplate(cell, offset, col, row, rowIndex, template) {
        /** @type {?} */
        const rowMap = this.outletRows[rowIndex] || {};
        /** @type {?} */
        const outlet = rowMap[col.name];
        if (!cell) {
            rowMap[col.name] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (cell !== outlet.targetElement) {
                this.updateOutlet(outlet, template, cell, AnjTable.rowContext(row, rowIndex, col));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex, col));
            }
        }
        else {
            rowMap[col.name] = this.createOutlet(cell, template, AnjTable.rowContext(row, rowIndex, col));
            this.outletRows[rowIndex] = rowMap;
        }
        col.initCell(cell, offset);
    }
    /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    rowTemplate(templateName, targetEl, row, rowIndex) {
        // Dispose row template no longer used for that index
        this.outletRows[rowIndex] = this.filterRowTemplates(this.outletRows[rowIndex], templateName);
        /** @type {?} */
        const outlet = this.outletRows[rowIndex][templateName];
        if (!targetEl) {
            this.outletRows[rowIndex][templateName] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable.rowContext(row, rowIndex));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex));
            }
        }
        else {
            this.outletRows[rowIndex][templateName] = this.createOutlet(targetEl, templateName, AnjTable.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = this.outletRows[rowIndex];
        }
    }
    /**
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} row
     * @param {?} rowIndex
     * @return {?}
     */
    fromTemplate(templateName, targetEl, row, rowIndex) {
        /** @type {?} */
        const rowMap = this.outletRows[rowIndex] || {};
        /** @type {?} */
        const outlet = rowMap[templateName];
        if (!targetEl) {
            rowMap[templateName] = outlet ? outlet.dispose() : undefined;
            return;
        }
        if (outlet) {
            if (targetEl !== outlet.targetElement) {
                this.updateOutlet(outlet, templateName, targetEl, AnjTable.rowContext(row, rowIndex));
            }
            else {
                AnjTable.updateContext(outlet, AnjTable.rowContext(row, rowIndex));
            }
        }
        else {
            rowMap[templateName] = this.createOutlet(targetEl, templateName, AnjTable.rowContext(row, rowIndex));
            this.outletRows[rowIndex] = rowMap;
        }
    }
    /**
     * @private
     * @param {?} target
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    createOutlet(target, templateName, context) {
        /** @type {?} */
        const outlet = new DomOutlet(target);
        outlet.attach(this.createTemplate(templateName, context));
        return outlet;
    }
    /**
     * @private
     * @param {?} templateName
     * @param {?} context
     * @return {?}
     */
    createTemplate(templateName, context) {
        /** @type {?} */
        const templateRef = this.templates.find(t => t.name === templateName);
        return new Template(templateRef, context);
    }
    /**
     * @private
     * @param {?} outlet
     * @param {?} templateName
     * @param {?} targetEl
     * @param {?} context
     * @return {?}
     */
    updateOutlet(outlet, templateName, targetEl, context) {
        outlet.targetElement = targetEl;
        outlet.detach();
        outlet.attach(this.createTemplate(templateName, context));
    }
    /**
     * Disposes of row templates no longer used in a given row. This ONLY happens if they have specified more than one
     * Template and the end user sorts or filters or otherwise changes the order. If only one template is specified,
     * the output will always match the input -r
     * @private
     * @param {?=} rowMap
     * @param {?=} templateName
     * @return {?}
     */
    filterRowTemplates(rowMap = {}, templateName) {
        /** @type {?} */
        const updated = {};
        Object.keys(rowMap).forEach(k => {
            if (k === templateName) {
                updated[k] = rowMap[k];
            }
            else {
                rowMap[k].dispose(false);
            }
        });
        return updated;
    }
}
AnjTable.decorators = [
    { type: Directive, args: [{
                selector: 'anj-table:not([anjIgnore])',
            },] }
];
/** @nocollapse */
AnjTable.ctorParameters = () => [
    { type: ElementRef }
];
AnjTable.propDecorators = {
    templates: [{ type: ContentChildren, args: [TemplateDirective,] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const TransitionState = {
    Enter: 0,
    Leave: 1,
    EnterTo: 2,
    Inactive: 3,
};
TransitionState[TransitionState.Enter] = 'Enter';
TransitionState[TransitionState.Leave] = 'Leave';
TransitionState[TransitionState.EnterTo] = 'EnterTo';
TransitionState[TransitionState.Inactive] = 'Inactive';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TransitionDirective {
    /**
     * @param {?} renderer
     * @param {?} element
     * @param {?} mode
     * @param {?} name
     */
    constructor(renderer, element, mode, name) {
        this.renderer = renderer;
        this.element = element;
        this.mode = mode;
        this.name = name;
        this.state = TransitionState.EnterTo;
        this.end = new EventEmitter();
        this.mode = this.mode ? this.mode : 'in-out';
        this.name = this.name ? this.name : 'transition';
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.endTransition();
        if (!changes.anjTransition.previousValue && !changes.anjTransition.currentValue && this.mode !== 'out') {
            this.initEnter();
        }
        else if (!changes.anjTransition.previousValue && changes.anjTransition.currentValue) {
            this.transitionIn();
        }
        else {
            this.transitionOut();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleTransition(event) {
        if (event.target === this.element.nativeElement && TransitionDirective.supportedTransitions.indexOf(event.propertyName) > -1) {
            event.stopPropagation();
            this.end.emit(this.state);
            if (this.state === TransitionState.Enter) {
                this.endTransition();
            }
            else {
                if (this.mode !== 'out' && this.anjTransition) {
                    this.transitionIn();
                }
                else {
                    this.endTransition();
                }
            }
        }
    }
    /**
     * @private
     * @return {?}
     */
    initEnter() {
        this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
    }
    /**
     * @private
     * @return {?}
     */
    transitionIn() {
        if (this.mode !== 'out') {
            this.state = TransitionState.Enter;
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave`);
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter`);
        }
        else {
            this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.endTransition();
        }
    }
    /**
     * @private
     * @return {?}
     */
    transitionOut() {
        this.state = TransitionState.Leave;
        if (this.mode !== 'in') {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-leave`);
        }
        else {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
            this.state = TransitionState.EnterTo;
            if (this.anjTransition) {
                window.setTimeout(() => {
                    this.transitionIn();
                });
            }
        }
    }
    /**
     * @private
     * @return {?}
     */
    endTransition() {
        this.state = TransitionState.Inactive;
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter-active`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave-active`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-enter`);
        this.renderer.removeClass(this.element.nativeElement, `anj-${this.name}-leave`);
        if (!this.anjTransition) {
            this.renderer.addClass(this.element.nativeElement, `anj-${this.name}-enter-to`);
        }
    }
}
TransitionDirective.supportedTransitions = ['opacity', 'transform'];
TransitionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[anjTransition]'
            },] }
];
/** @nocollapse */
TransitionDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: undefined, decorators: [{ type: Attribute, args: ['mode',] }] },
    { type: String, decorators: [{ type: Attribute, args: ['name',] }] }
];
TransitionDirective.propDecorators = {
    anjTransition: [{ type: Input }],
    end: [{ type: Output }],
    handleTransition: [{ type: HostListener, args: ['transitiend', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ValueAccessor {
    /**
     * @param {?} element
     * @param {?} parentForm
     * @param {?} parentGroup
     * @param {?} injector
     * @param {?} changeRef
     */
    constructor(element, parentForm, parentGroup, injector, changeRef) {
        this.element = element;
        this.injector = injector;
        this.changeRef = changeRef;
        /**
         * Enable two-way binding
         */
        this.valueChange = new EventEmitter();
        this.onChange = () => { };
        this.onTouched = () => { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
    }
    /**
     * @return {?}
     */
    get isInvalid() {
        if (!this.ngControl) {
            return false;
        }
        /** @type {?} */
        const isSubmitted = (this.parent && this.parent.submitted);
        return this.ngControl.invalid && (this.ngControl.touched || isSubmitted);
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        /** @type {?} */
        const control = this.injector.get(NgControl, null);
        if (control) {
            this.ngControl = (/** @type {?} */ (control.control));
            this.element.nativeElement.validationStrategy = this;
        }
        if (this.parent && this.parent.statusChanges) {
            this.parentSubscription = this.parent.statusChanges.subscribe({
                next: () => {
                    this.changeRef.markForCheck();
                }
            });
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.parentSubscription) {
            this.parentSubscription.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    onAnjBlur() {
        this.onTouched();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.element.nativeElement.value = this.lastValue = value == null ? '' : value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onChangeEvent(value) {
        if (value !== this.lastValue) {
            this.lastValue = value;
            this.onChange(value);
            this.valueChange.emit(value);
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.element.nativeElement.disabled = isDisabled;
    }
    // Called by Anjuna Element, so needs to exist, but Angular does all the hoof-work
    /**
     * @return {?}
     */
    onValidChange() { }
}
/** @nocollapse */
ValueAccessor.ctorParameters = () => [
    { type: ElementRef },
    { type: NgForm },
    { type: FormGroupDirective },
    { type: Injector },
    { type: ChangeDetectorRef, decorators: [{ type: Optional }] }
];
ValueAccessor.propDecorators = {
    isInvalid: [{ type: HostBinding, args: ['attr.invalid',] }],
    valueChange: [{ type: Output }],
    onAnjBlur: [{ type: HostListener, args: ['anjBlur',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class BooleanValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     * @param {?} injector
     * @param {?} changeRef
     */
    constructor(parentForm, parentFormGroup, element, injector, changeRef) {
        super(element, parentForm, parentFormGroup, injector, changeRef);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.element.nativeElement.checked = this.lastValue = value == null ? false : value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onAnjChange(value) {
        this.onChangeEvent(value);
    }
}
BooleanValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-checkbox, anj-toggle',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: BooleanValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
BooleanValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef },
    { type: Injector },
    { type: ChangeDetectorRef }
];
BooleanValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target.checked'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@dynamic
 * @abstract
 */
class AnjunaValidators {
    /**
     * @param {?} c
     * @return {?}
     */
    static date(c) {
        if (!c.value) {
            return;
        }
        /** @type {?} */
        let isValid = true;
        // Handle obvious Invalid Date
        if (c.value.toString() === 'Invalid Date') {
            isValid = false;
            // Handle Moment Date
        }
        else if (typeof c.value._isValid === 'boolean') {
            isValid = c.value._isValid;
            // Handle Native Date
        }
        else {
            isValid = !isNaN(new Date(c.value).getTime());
        }
        if (isValid || c.value === '') {
            return null;
        }
        else {
            return {
                date: {
                    valid: false
                }
            };
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    static requiredCheck(c) {
        if (c.value) {
            return null;
        }
        else {
            return {
                required: {
                    valid: false
                }
            };
        }
    }
    /**
     * @param {?} c
     * @return {?}
     */
    static unique(c) {
        if (!c.value || !Array.isArray(c.value)) {
            return;
        }
        /** @type {?} */
        const testArray = c.value.filter(Boolean);
        /** @type {?} */
        const isValid = testArray.length === Array.from(new Set(testArray)).length;
        if (isValid) {
            return null;
        }
        else {
            return {
                unique: {
                    valid: false
                }
            };
        }
    }
}
AnjunaValidators.minDate = (min) => {
    return (c) => {
        if (!c.value) {
            return;
        }
        if (c.value < new Date(min)) {
            return {
                min: {
                    valid: false
                }
            };
        }
        else {
            return null;
        }
    };
};
AnjunaValidators.maxDate = (max) => {
    return (c) => {
        if (!c.value) {
            return;
        }
        if (c.value > new Date(max)) {
            return {
                max: {
                    valid: false
                }
            };
        }
        else {
            return null;
        }
    };
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class DateValidatorDirective {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return AnjunaValidators.date(c);
    }
}
DateValidatorDirective.decorators = [
    { type: Directive, args: [{
                selector: '[date][ngModel],[date][formControl]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: DateValidatorDirective, multi: true }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TIME_TIMEZONE_STRING = new InjectionToken('TimeTimezoneString');
/** @type {?} */
const TIME_MERIDIEM = new InjectionToken('TimeMeridiem');
/** @type {?} */
const DATE_TIMEZONE_OVERRIDE = new InjectionToken('DateTimezoneOverride');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DatePickerDirective {
    /**
     * @param {?} renderer
     * @param {?} _element
     * @param {?} changeRef
     * @param {?=} _locale
     * @param {?=} _timezone
     * @param {?=} ngControl
     */
    constructor(renderer, _element, changeRef, _locale, _timezone, ngControl) {
        this.renderer = renderer;
        this._element = _element;
        this.changeRef = changeRef;
        this._locale = _locale;
        this._timezone = _timezone;
        this.ngControl = ngControl;
        this.onChange = () => { };
        this.onTouch = () => { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    // Custom input to return inner value, but display masked value
    /**
     * @return {?}
     */
    get value() { return this.innerDateValue; }
    /**
     * @param {?} dateValue
     * @return {?}
     */
    set value(dateValue) {
        if (dateValue) {
            this.outerValue = this.prepDateStringOutput(dateValue);
            this.changeRef.markForCheck();
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.picker = this._element.nativeElement.querySelector('anj-date-picker');
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.value = this.ngModel || this.ngControl.value;
            this.changeRef.detectChanges();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.listenerFn) {
            this.listenerFn();
        }
    }
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.value = newValue;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        if (event.target.tagName === 'ANJ-INPUT') {
            this.onTouch();
            /** @type {?} */
            const value = (this.picker && this.picker.adapter) ? this.picker.adapter.processDateInput(event.target.value) : event.target.value;
            this.updateInnerValue(value);
            this.valueChange.emit(this.innerDateValue);
            this.changeRef.markForCheck();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onCalendarSelect(event) {
        this.onTouch();
        this.updateInnerValue(event.detail);
        this.valueChange.emit(this.innerDateValue);
        this.changeRef.markForCheck();
    }
    // convert the masked value to storage value
    /**
     * @private
     * @param {?} inputDate
     * @return {?}
     */
    updateInnerValue(inputDate) {
        this.innerDateValue = this.picker.adapter.convertToDate(inputDate);
        this.propogateInnerValueChange();
    }
    /**
     * @private
     * @return {?}
     */
    propogateInnerValueChange() {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
        this.valueChange.emit(this.innerDateValue);
    }
    /**
     * @private
     * @param {?} input
     * @return {?}
     */
    prepDateStringOutput(input) {
        if (!input) {
            return null;
        }
        else if (typeof input === 'string') {
            return input;
        }
        else if (this._timezone) {
            // Timezone-override aware formatting
            /** @type {?} */
            const tzTime = new Date(input).toLocaleString(undefined, { timeZone: this._timezone });
            return new Date(tzTime).toLocaleDateString(this._locale);
        }
        else {
            return new Date(input).toLocaleDateString(this._locale, { month: '2-digit', day: '2-digit', year: 'numeric' });
        }
    }
}
DatePickerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjDatePicker]',
                exportAs: 'anjDatepickerInput'
            },] }
];
/** @nocollapse */
DatePickerDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [DATE_TIMEZONE_OVERRIDE,] }] },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
];
DatePickerDirective.propDecorators = {
    value: [{ type: Input }],
    ngModel: [{ type: Input }],
    outerValue: [{ type: HostBinding, args: ['value',] }],
    onInput: [{ type: HostListener, args: ['anjBlur', ['$event'],] }],
    onCalendarSelect: [{ type: HostListener, args: ['anjCalendarSelection', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class TextValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     * @param {?} injector
     * @param {?} changeRef
     */
    constructor(parentForm, parentFormGroup, element, injector, changeRef) {
        super(element, parentForm, parentFormGroup, injector, changeRef);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onAnjChange(value) {
        this.onChangeEvent(value);
    }
}
TextValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input, anj-textarea',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: TextValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
TextValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef },
    { type: Injector },
    { type: ChangeDetectorRef }
];
TextValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.detail.value'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class RequiredCheckDirective {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return AnjunaValidators.requiredCheck(c);
    }
}
RequiredCheckDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-checkbox[required][ngModel], anj-toggle[required][ngModel]',
                providers: [
                    { provide: NG_VALIDATORS, useExisting: RequiredCheckDirective, multi: true }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class SelectValueDirective extends ValueAccessor {
    /**
     * @param {?} parentForm
     * @param {?} parentFormGroup
     * @param {?} element
     * @param {?} injector
     * @param {?} changeRef
     */
    constructor(parentForm, parentFormGroup, element, injector, changeRef) {
        super(element, parentForm, parentFormGroup, injector, changeRef);
    }
    /**
     * @param {?} target
     * @return {?}
     */
    onAnjChange(target) {
        // ignore the input for type-aheads
        if (target.matches('anj-input')) {
            return;
        }
        this.onChangeEvent(target.value);
    }
}
SelectValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-radio-group, anj-select, anj-type-ahead',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: SelectValueDirective,
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
SelectValueDirective.ctorParameters = () => [
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: ElementRef },
    { type: Injector },
    { type: ChangeDetectorRef }
];
SelectValueDirective.propDecorators = {
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TimePickerDirective {
    /**
     * @param {?} _element
     * @param {?=} ngControl
     * @param {?=} _timezone
     * @param {?=} _meridiem
     */
    constructor(_element, ngControl, _timezone, _meridiem) {
        this._element = _element;
        this.ngControl = ngControl;
        this._timezone = _timezone;
        this._meridiem = _meridiem;
        this.onChange = () => { };
        this.onTouch = () => { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        /** @type {?} */
        const picker = this._element.nativeElement.querySelector('anj-time-picker');
        picker.setAttribute('meridiem', this._meridiem);
        if (this._timezone) {
            picker.setAttribute('timezone', this._timezone);
        }
    }
    // // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.value = newValue;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        this.value = event.detail;
        if (this.ngControl) {
            this.ngControl.control.setValue(this.value);
        }
        this.valueChange.emit(this.value);
    }
}
TimePickerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjTimePicker]',
                exportAs: 'anjTimepickerInput'
            },] }
];
/** @nocollapse */
TimePickerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [TIME_TIMEZONE_STRING,] }] },
    { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
];
TimePickerDirective.propDecorators = {
    value: [{ type: HostBinding, args: ['value',] }],
    onInput: [{ type: HostListener, args: ['anjClockSelection', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ZuoraDatePipe {
    /**
     * @param {?=} _locale
     */
    constructor(_locale) {
        this._locale = _locale;
    }
    /**
     * @param {?} value
     * @param {?=} args
     * @return {?}
     */
    transform(value, args) {
        try {
            /** @type {?} */
            const valueAsDate = new Date(value);
            /** @type {?} */
            const userOpts = (/** @type {?} */ (args));
            /** @type {?} */
            const defaultOpts = { year: '2-digit', month: 'numeric', day: 'numeric' };
            /** @type {?} */
            const opts = Object.assign({}, defaultOpts, userOpts);
            return new Intl.DateTimeFormat(this._locale, opts).format(valueAsDate);
        }
        catch (e) {
            return value.toString();
        }
    }
}
ZuoraDatePipe.decorators = [
    { type: Pipe, args: [{
                name: 'zuoraDate'
            },] }
];
/** @nocollapse */
ZuoraDatePipe.ctorParameters = () => [
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ToastService {
    constructor() {
        this._dismissed$ = new Subject();
    }
    /**
     * @return {?}
     */
    get dismissed$() {
        return this._dismissed$.asObservable();
    }
    /**
     * @return {?}
     */
    static createController() {
        /** @type {?} */
        const controller = document.createElement('anj-toast-controller');
        document.body.appendChild(controller);
        return (/** @type {?} */ (controller));
    }
    /**
     * @return {?}
     */
    dismissAll() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.dismissAll();
        });
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    error(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.error(message, header);
        });
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    message(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.message(message, header);
        });
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    success(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.success(message, header);
        });
    }
    /**
     * @param {?} message
     * @param {?=} header
     * @return {?}
     */
    warn(message, header) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const controller = yield this.getController();
            return controller.warn(message, header);
        });
    }
    /**
     * @return {?}
     */
    getController() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.controller || this.findController();
        });
    }
    /**
     * @private
     * @return {?}
     */
    findController() {
        /** @type {?} */
        const controller = this.findOrCreate(document.querySelector('anj-toast-controller'));
        controller.addEventListener('anjToastDidDismiss', this.onToastDidDismiss.bind(this));
        this.controller = controller;
        return controller.componentOnReady();
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    onToastDidDismiss(event) {
        this._dismissed$.next(event);
    }
    /**
     * @private
     * @param {?} control
     * @return {?}
     */
    findOrCreate(control) {
        return control || ToastService.createController();
    }
}
ToastService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ ToastService.ngInjectableDef = defineInjectable({ factory: function ToastService_Factory() { return new ToastService(); }, token: ToastService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LocalStorageService extends LocalStorage {
}
LocalStorageService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ LocalStorageService.ngInjectableDef = defineInjectable({ factory: function LocalStorageService_Factory() { return new LocalStorageService(); }, token: LocalStorageService, providedIn: "root" });
class SessionStorageService extends SessionStorage {
}
SessionStorageService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */ SessionStorageService.ngInjectableDef = defineInjectable({ factory: function SessionStorageService_Factory() { return new SessionStorageService(); }, token: SessionStorageService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class LocaleDirective {
    /**
     * @param {?} changeRef
     * @param {?=} _locale
     * @param {?=} _lang
     * @param {?=} _timezone
     */
    constructor(changeRef, _locale, _lang, _timezone) {
        this.changeRef = changeRef;
        if (_locale) {
            this._locale = _locale;
            this._lang = _lang ? _lang : this._locale ? this._locale.split('-')[0] : 'en';
        }
        if (_timezone) {
            this._timezone = _timezone;
        }
        this.changeRef.markForCheck();
    }
}
LocaleDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-locale',
            },] }
];
/** @nocollapse */
LocaleDirective.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LANGUAGE_CODE,] }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [DATE_TIMEZONE_OVERRIDE,] }] }
];
LocaleDirective.propDecorators = {
    _locale: [{ type: HostBinding, args: ['attr.locale',] }],
    _lang: [{ type: HostBinding, args: ['attr.lang',] }],
    _timezone: [{ type: HostBinding, args: ['attr.timezone',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DatetimePickerDirective {
    /**
     * @param {?} _element
     * @param {?} changeRef
     * @param {?=} ngControl
     * @param {?=} _locale
     * @param {?=} _meridiem
     */
    constructor(_element, changeRef, ngControl, _locale, _meridiem) {
        this._element = _element;
        this.changeRef = changeRef;
        this.ngControl = ngControl;
        this._locale = _locale;
        this._meridiem = _meridiem;
        this.onChange = () => { };
        this.onTouch = () => { };
        this.valueChange = new EventEmitter();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.picker = this._element.nativeElement.querySelector('anj-datetime-picker');
        this.picker.setAttribute('meridiem', this._meridiem);
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.innerDateValue = this.ngModel || this.ngControl.value;
            /** @type {?} */
            const d = new Date(this.innerDateValue);
            this.outerValue = this.localizeForOuterValue(d);
            this.changeRef.detectChanges();
        }
    }
    // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.innerDateValue = newValue;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // End: ControlValueAccessor requirements
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        if (event.target.tagName === 'ANJ-INPUT') {
            this.picker.processInputValue(event.target.value);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onAnjDatetimeChange(event) {
        this.changeHandler(event.detail);
    }
    /**
     * @private
     * @param {?} changedVal
     * @return {?}
     */
    changeHandler(changedVal) {
        this.onTouch();
        this.outerValue = changedVal.outer;
        this.innerDateValue = changedVal.inner;
        this.propogateInnerValueChange();
        this.changeRef.markForCheck();
    }
    /**
     * @private
     * @param {?} date
     * @return {?}
     */
    localizeForOuterValue(date) {
        return date.toLocaleString(this._locale, { hour12: !!this._meridiem, month: '2-digit', day: '2-digit', year: 'numeric', hour: '2-digit', minute: '2-digit' });
    }
    /**
     * @private
     * @return {?}
     */
    propogateInnerValueChange() {
        if (this.ngControl) {
            this.ngControl.control.setValue(this.innerDateValue);
        }
    }
}
DatetimePickerDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-input[anjDatetimePicker]',
                exportAs: 'anjDatetimePickerInput'
            },] }
];
/** @nocollapse */
DatetimePickerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: String, decorators: [{ type: Optional }, { type: Inject, args: [LOCALE_ID,] }] },
    { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [TIME_MERIDIEM,] }] }
];
DatetimePickerDirective.propDecorators = {
    ngModel: [{ type: Input }],
    outerValue: [{ type: HostBinding, args: ['value',] }],
    onInput: [{ type: HostListener, args: ['anjBlur', ['$event'],] }],
    onAnjDatetimeChange: [{ type: HostListener, args: ['anjDatetimeChange', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/* tslint:disable:directive-selector */
class ListValueDirective {
    /**
     * @param {?} _element
     * @param {?} changeRef
     * @param {?} parentForm
     * @param {?} parentGroup
     * @param {?=} ngControl
     */
    constructor(_element, changeRef, parentForm, parentGroup, ngControl) {
        this._element = _element;
        this.changeRef = changeRef;
        this.ngControl = ngControl;
        this.valueChange = new EventEmitter();
        this.onChange = () => { };
        this.onTouch = () => { };
        this.parent = parentForm ? parentForm : parentGroup ? parentGroup : null;
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get parsedField() {
        return (this.field) ? this.field.parsedList : undefined;
    }
    /**
     * @return {?}
     */
    get list() {
        if (!this._list) {
            this.list = this._element.nativeElement;
        }
        return this._list;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    set list(el) {
        this._list = el;
    }
    /**
     * @return {?}
     */
    onAnjBlur() {
        this.onTouch();
    }
    /**
     * @param {?} target
     * @return {?}
     */
    onAnjChange(target) {
        if (!target.matches('anj-list')) {
            return;
        }
        this.changeHandler(target.value.slice(0));
    }
    /**
     * @return {?}
     */
    get isInvalid() {
        if (this.ngControl) {
            /** @type {?} */
            const isSubmitted = (this.parent && this.parent.submitted);
            return (this.ngControl.invalid && (this.ngControl.touched || isSubmitted));
        }
        if (this.control) {
            return this.control.invalid;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if ((this.ngControl && this.ngControl.value) || this.ngModel) {
            this.value = this.ngModel || this.ngControl.value;
            this.list.value = this.value;
            this.changeRef.detectChanges();
        }
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.setAttributes();
        if (this.control && this.control.value) {
            this.value = this.control.value;
            this.list.value = this.value;
            this.changeRef.detectChanges();
        }
    }
    /**
     * @return {?}
     */
    setAttributes() {
        if (this.field) {
            this.list.setAttribute('min', this.field.minItems || this.list.min);
            this.list.setAttribute('max', this.field.maxItems || this.list.max);
        }
        if (this.parsedField) {
            this.list.setAttribute('options', this.parsedField.options || this.list.options);
            this.list.setAttribute('type', this.parsedField.toInputType() || this.list.type);
        }
    }
    // Start: ControlValueAccessor requirements
    /**
     * @param {?} newValue
     * @return {?}
     */
    writeValue(newValue) {
        this.value = newValue;
        this.list.value = this.value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    // End: ControlValueAccessor requirements
    /**
     * @private
     * @param {?} changedVal
     * @return {?}
     */
    changeHandler(changedVal) {
        this.onTouch();
        this.value = changedVal;
        this.valueChange.emit(this.value);
        this.propogateValueChange();
    }
    /**
     * @private
     * @return {?}
     */
    propogateValueChange() {
        if (this.ngControl) {
            this.ngControl.control.patchValue(this.value);
        }
        else if (this.control) {
            this.control.controls = [];
            this.value.map(value => this.control.push(new FormControl(value)));
        }
        this.changeRef.detectChanges();
    }
}
ListValueDirective.decorators = [
    { type: Directive, args: [{
                selector: 'anj-list'
            },] }
];
/** @nocollapse */
ListValueDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
];
ListValueDirective.propDecorators = {
    ngModel: [{ type: Input }],
    control: [{ type: Input }],
    field: [{ type: Input }],
    valueChange: [{ type: Output }],
    onAnjBlur: [{ type: HostListener, args: ['anjBlur',] }],
    onAnjChange: [{ type: HostListener, args: ['anjChange', ['$event.target'],] }],
    isInvalid: [{ type: HostBinding, args: ['attr.invalid',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DECLARATIONS = [
    // Common
    DraggableDirective,
    DroppableDirective,
    LocaleDirective,
    TemplateDirective,
    TransitionDirective,
    ZuoraDatePipe,
    // Forms
    BooleanValueDirective,
    DatePickerDirective,
    DatetimePickerDirective,
    DateValidatorDirective,
    ListValueDirective,
    RequiredCheckDirective,
    SelectValueDirective,
    TextValueDirective,
    TimePickerDirective,
    // Tables
    AnjTable
];
class AnjunaCoreModule {
}
AnjunaCoreModule.decorators = [
    { type: NgModule, args: [{
                declarations: DECLARATIONS,
                exports: DECLARATIONS,
                providers: [
                    {
                        provide: TIME_MERIDIEM,
                        useValue: true
                    },
                    ZuoraDatePipe
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BaseTranslationPipe, DraggableDirective, DroppableDirective, AnjTable, TemplateDirective, TransitionDirective, BooleanValueDirective, DateValidatorDirective, DatePickerDirective, TextValueDirective, RequiredCheckDirective, SelectValueDirective, TimePickerDirective, LANGUAGE_CODE, TIME_TIMEZONE_STRING, TIME_MERIDIEM, DATE_TIMEZONE_OVERRIDE, DomOutlet, Template, AnjunaValidators, ZuoraDatePipe, ToastService, LocalStorageService, SessionStorageService, TransitionState, AnjunaCoreModule, LocaleDirective as ɵb, DatetimePickerDirective as ɵc, ListValueDirective as ɵd, ValueAccessor as ɵa };

//# sourceMappingURL=anjuna-angular-core.js.map